@@ -8,7 +8,7 @@ import logging
 import os
 import sys
 
-from .recommend import BGARecommender, BGGRecommender
+from board_game_recommender.recommend import BGARecommender, BGGRecommender
 
 LOGGER = logging.getLogger(__name__)
 BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
