@@ -5,17 +5,17 @@
 import csv
 import logging
 import os
-import tempfile
 import sys
+import tempfile
 
 # from datetime import date
-from typing import Any, Dict, Optional, Tuple, Type
-
-from pytility import arg_to_iter, clear_list
+from typing import Any, Dict, FrozenSet, Iterable, Optional, Tuple, Type
 
 import turicreate as tc
+from pytility import arg_to_iter, clear_list
 
-from .utils import (
+from board_game_recommender.base import BaseGamesRecommender, GameKeyType, UserKeyType
+from board_game_recommender.utils import (
     condense_csv,
     filter_sframe,
     format_from_path,
@@ -82,7 +82,7 @@ def make_cluster(data, item_id, target, target_dtype=str):
     return clusters.filter(lambda x: x is not None and len(x) > 1)
 
 
-class GamesRecommender:
+class GamesRecommender(BaseGamesRecommender):
     """games recommender"""
 
     logger = logging.getLogger("GamesRecommender")
@@ -113,7 +113,7 @@ class GamesRecommender:
     _cooperatives = None
 
     def __init__(
-        self,
+        self: "GamesRecommender",
         model,
         similarity_model=None,
         games=None,
@@ -134,7 +134,7 @@ class GamesRecommender:
             self._compilations = compilations
 
     @property
-    def rated_games(self):
+    def rated_games(self: "GamesRecommender") -> FrozenSet[GameKeyType]:
         """rated games"""
         if self._rated_games is None:
             self._rated_games = frozenset(
@@ -143,7 +143,7 @@ class GamesRecommender:
         return self._rated_games
 
     @property
-    def known_games(self):
+    def known_games(self: "GamesRecommender") -> FrozenSet[GameKeyType]:
         """known games"""
         if self._known_games is None:
             self._known_games = (
@@ -154,7 +154,7 @@ class GamesRecommender:
         return self._known_games
 
     @property
-    def known_users(self):
+    def known_users(self: "GamesRecommender") -> FrozenSet[UserKeyType]:
         """known users"""
         if self._known_users is None:
             self._known_users = frozenset(
@@ -165,14 +165,14 @@ class GamesRecommender:
         return self._known_users
 
     @property
-    def num_games(self):
+    def num_games(self: "GamesRecommender") -> int:
         """total number of games known to the recommender"""
         if self._num_games is None:
             self._num_games = len(self.known_games)
         return self._num_games
 
     @property
-    def clusters(self):
+    def clusters(self: "GamesRecommender"):
         """game implementation clusters"""
         if self._clusters is None:
             self._clusters = make_cluster(
@@ -184,7 +184,7 @@ class GamesRecommender:
         return self._clusters
 
     @property
-    def compilations(self):
+    def compilations(self: "GamesRecommender"):
         """compilation games"""
         if self._compilations is None:
             self._compilations = (
@@ -197,7 +197,7 @@ class GamesRecommender:
         return self._compilations
 
     @property
-    def cooperatives(self):
+    def cooperatives(self: "GamesRecommender"):
         """cooperative games"""
         if self._cooperatives is None:
             self._cooperatives = (
@@ -209,11 +209,11 @@ class GamesRecommender:
             )
         return self._cooperatives
 
-    def filter_games(self, **filters):
+    def filter_games(self: "GamesRecommender", **filters):
         """return games filtered by given criteria"""
         return filter_sframe(self.games, **filters)
 
-    def cluster(self, game_id):
+    def cluster(self: "GamesRecommender", game_id):
         """get implementation cluster for a given game"""
 
         # pylint: disable=len-as-condition
@@ -230,7 +230,7 @@ class GamesRecommender:
 
         return self._game_clusters.get(game_id) or (game_id,)
 
-    def _process_games(self, games=None, games_filters=None):
+    def _process_games(self: "GamesRecommender", games=None, games_filters=None):
         games = (
             games[self.id_field].astype(self.id_type, True)
             if isinstance(games, tc.SFrame)
@@ -267,7 +267,7 @@ class GamesRecommender:
         return games
 
     def _process_exclude(
-        self,
+        self: "GamesRecommender",
         users,
         exclude=None,
         exclude_known=True,
@@ -323,7 +323,7 @@ class GamesRecommender:
         return exclude
 
     def _post_process_games(
-        self,
+        self: "GamesRecommender",
         games,
         columns,
         join_on=None,
@@ -347,13 +347,14 @@ class GamesRecommender:
         return games.sort(sort_by, ascending=ascending)[columns]
 
     # pylint: disable=no-self-use
-    def process_user_id(self, user_id):
+    def process_user_id(self: "GamesRecommender", user_id):
         """process user ID"""
         return user_id or None
 
     def recommend(
-        self,
-        users=None,
+        self: "GamesRecommender",
+        users: Iterable[UserKeyType],
+        *,
         similarity_model=False,
         games=None,
         games_filters=None,
@@ -366,7 +367,7 @@ class GamesRecommender:
         columns=None,
         star_percentiles=None,
         **kwargs,
-    ):
+    ) -> tc.SFrame:
         """recommend games"""
 
         users = [self.process_user_id(user) for user in arg_to_iter(users)] or [None]
@@ -430,15 +431,16 @@ class GamesRecommender:
         )
 
     def recommend_similar(
-        self,
-        games=None,
+        self: "GamesRecommender",
+        games: Iterable[GameKeyType],
+        *,
         items=None,
         games_filters=None,
         threshold=0.001,
         num_games=None,
         columns=None,
         **kwargs,
-    ):
+    ) -> tc.SFrame:
         """recommend games similar to given ones"""
 
         games = list(arg_to_iter(games))
@@ -469,7 +471,14 @@ class GamesRecommender:
             games=recommendations, columns=columns, join_on=self.id_field
         )
 
-    def similar_games(self, games, num_games=10, columns=None):
+    def similar_games(
+        self: "GamesRecommender",
+        games: Iterable[GameKeyType],
+        *,
+        num_games=10,
+        columns=None,
+        **kwargs,
+    ) -> tc.SFrame:
         """find similar games"""
 
         games = list(arg_to_iter(games))
@@ -494,7 +503,7 @@ class GamesRecommender:
         )
 
     def lead_game(
-        self,
+        self: "GamesRecommender",
         game_id,
         user=None,
         exclude_known=False,
@@ -537,7 +546,7 @@ class GamesRecommender:
         return ranked[self.id_field][0] if ranked else game_id
 
     def save(
-        self,
+        self: "GamesRecommender",
         path,
         dir_model="recommender",
         dir_similarity="similarity",
